/*

                       Inertia Realtime 3D Rendering Engine
            Copyright (c) 1997, Alex Chalfin. All Rights Reserved.
           DISTRIBUTION OF THIS SOURCE CODE IS STRICTLY PROHIBITED

*/


#include "pcx.h"

#pragma pack (1)
typedef struct
{
  char  Manufacturer;
  char  Version;
  char  Encoding;
  char  BitsPerPixel;
  unsigned short int XMin;
  unsigned short int YMin;
  unsigned short int XMax;
  unsigned short int YMax;
  unsigned short int HDpi;
  unsigned short int VDpi;
  char  Colormap[48];
  char  Reserved;
  char  NPlanes;
  unsigned short int BytesPerLine;
  unsigned short int PaletteInfo;
  unsigned short int HScreenSize;
  unsigned short int VScreenSize;
  char  Filler[54];
} PCXHeader;


int LoadPCXFile(char *Filename, char **Bitmap, char *Palette, int *Size, int FOfs)
{
  PCXHeader Header;
  FILE      *fp;
  int    XSize;
  int    YSize;
  char     Run;
  char     Value;
  int    i;
  int    xc;

  fp = fopen(Filename, "rb");
  if (fp == NULL)
    return(I_FILEERR);

  fseek(fp, FOfs, SEEK_SET);

  if (fread(&Header, sizeof(Header), 1, fp) != 1)
  {
    fclose(fp);
    return(I_FILEERR);
  }

  if ((Header.Manufacturer != 0x0a) && (Header.Encoding != 0x01))
  {
    fclose(fp);
    return(I_FILEERR);
  }

  if (Header.BitsPerPixel != 8)
    {
      fclose(fp);
      return(I_FILEERR);
    }

  XSize = Header.XMax - Header.XMin + 1;
  YSize = Header.YMax - Header.YMin + 1;

  *Size = XSize * YSize;

  *Bitmap = (char *)malloc(XSize * YSize);

  if ((*Bitmap) == NULL)
  {
    fclose(fp);
    return(I_FILEERR);
  }

  for (i = 0; i < YSize; i++)
  {
    xc = 0;
    while (xc < XSize) /* loop for a single line */
      {
        fread(&Run, sizeof(Run), 1, fp);
        if ((Run & 0xc0) == 0xc0)   /* see if we have a run or a color */
          {
            Run = Run & 0x3f;  /* count */
            fread(&Value, sizeof(Value), 1, fp);
          }
        else
          {
            Value = Run;
            Run = 1;
          }

        while ((xc < XSize) && (Run > 0))
          {
            (*Bitmap)[i*XSize+xc] = Value;
            xc++;
            Run--;
          }
        }
      }

    do  /* read to the palette */
      {
        fread(&Value, sizeof(Value), 1, fp);
      } while (Value != 0x0c);

  fread(Palette, 256 * 3 * sizeof(char), 1, fp);

  fclose(fp);

  for (i = 0; i < 256; i++)
    {
      Palette[i*3] >>= 2;
      Palette[i*3+1] >>= 2;
      Palette[i*3+2] >>= 2;
    }

  return(I_OK);
}


int SavePCX(char *Filename, int x, int y, char *Bitmap, char *Palette)
{
  return(0);
}