;
;                    Inertia Realtime 3D Rendering Engine
;    Copyright (c) 1996, Alex Chalfin, Jeroen Bouwens. All Rights Reserved.
;          DISTRIBUTION OF THIS SOURCE CODE IS STRICTLY PROHIBITED
;

;
; POLYGOTX.ASM
; code: Alex Chalfin
;

; Gouraud Texture (affine) polygon filler
;
;================================================================
; Polygon specific defines
;================================================================
;
__INTERP_EXTRA__  EQU 0        ; must be defined for everything but flat
__INTERP_C__      EQU 0        ; must be defined for goruaud shading
__INTERP_UV__     EQU 0        ; must be defined for texture mapping
;__INTERP_ENV__    EQU 0        ; must be defined for env mapping (phong)
;

.386p                           ; 486 protected mode
.387
.Model Flat,C                   ; with flat memory model

DATA SEGMENT PARA PUBLIC USE32  ;
                                ;
public _GouraudTexturePolygonASM ;
                                ;
include strucdef.asm            ; Inertia structure references
include varsasm.inc             ; external variable references
                                ;
DATA ENDS                       ;
                                ;
;================================================================
; Code Segment
;================================================================
                                ;
CODE SEGMENT PARA PUBLIC USE32  ;
                                ;
;===========================================================================
; Macros
;===========================================================================
                                ;
                                ; All registers are available for loop
                                ; macro. Nothing needs to be saved/restored
                                ; it is all handled in AFFASM.INC
                                ;
INNER_LOOP Macro  Left, Right, Height, Side;
Local  @@HeightLooper,@@ExitHeight,@@SkipFill,@@FillLoop ; Local labels
                                ;
 @@HeightLooper:                ;
  Mov  eax,[Height]             ;
  Cmp  eax,0                    ;
  Jle @@ExitHeight              ;
                                ;
  Dec  [Height]                 ;  decrement the height value by 1
                                ;
  Mov  ebx,[Left&X]             ;
  Add  ebx,65535                ;
  And  ebx,0ffff0000h           ;
                                ;
  Mov  edx,ebx                  ;
  Sub  edx,[Left&X]             ;
                                ; edx contains sub-pixel correction factor
  Mov  esi,edx                  ;
                                ;
  Mov  eax,[_HorizontalDU]       ;
  IMul edx                      ;
  Shrd eax,edx,16               ; fix for sub-pixel adjustment
                                ;
  Add  eax,[Side&U]             ; eax is the starting horizontal color
                                ;
  Mov  [_UStart],eax             ;
                                ;
  Mov  edx,esi                  ; get sup-pixel correction factor
  Mov  eax,[_HorizontalDV]       ;
  IMul edx                      ;
  Shrd eax,edx,16               ; fix for sub-pixel adjustment
                                ;
  Add  eax,[Side&V]             ; eax is the starting horizontal color
                                ;
  Mov  [_VStart],eax             ;
                                ;
  Mov  edx,esi                  ; get sup-pixel correction factor
  Mov  eax,[_HorizontalDC]       ;
  IMul edx                      ;
  Shrd eax,edx,16               ; fix for sub-pixel adjustment
                                ;
  Add  eax,[Side&C]             ; eax is the starting horizontal color
                                ;
  Mov  [_CStart],eax             ;
                                ;
  Sar  ebx,16                   ;
                                ;
  Mov  ecx,[Right&X]            ;
  Dec  ecx                      ;
  Sar  ecx,16                   ;
                                ;
  Sub  ecx,ebx                  ;
  Inc  ecx                      ;
  Cmp  ecx,0                    ;
  Jle @@SkipFill                ;
                                ;
  Mov  [_Count],ecx              ;
                                ;
  Mov  edi,[_YAddress]           ;
  Add  edi,ebx                  ;
                                ;
  Mov  eax,[_UStart]             ;
  Mov  ebx,[_VStart]             ;
                                ;
  Sar  eax,8                    ;
  Sar  ebx,8                    ;
                                ;
  Mov  edx,[_HorizontalDU]       ;
  Mov  esi,[_HorizontalDV]       ;
                                ;
  Sar  edx,8                    ;
  Sar  esi,8                    ;
                                ;
  Mov  [_HDU8],edx               ;
  Mov  [_HDV8],esi               ;
  Mov  esi,[_CStart]             ;
                                ;
  Xor  edx,edx                  ;
  Xor  ecx,ecx                  ;
                                ; add in the intensity offset
                                ; On entering loop:
                                ; edi = screen position
                                ; esi = 16.16 gouraud intensity
                                ; eax = 8.8 texture U value
                                ; ebx = 8.8 texture V value
  Push  ebp                     ;
  Mov  ebp,[_TextureOfsTable]    ;
 @@FillLoop:                    ;
  Xor  edx,edx                  ;
                                ;
  Mov  dl,ah                    ;
  Mov  dh,bh                    ;
                                ;
  Add  eax,[_HDU8]               ;
  Add  ebx,[_HDV8]               ;
                                ;
  Mov  cl,[ebp+edx]             ; get texture color
                                ;
  Mov  edx,esi                  ;
  Add  ecx,[_IntensityTable]     ;
                                ;
  And  edx,003f0000h            ;
  Add  esi,[_HorizontalDC]       ;
                                ;
  Shr  edx,8                    ;
  Inc  edi                      ;
                                ;
  Add  ecx,edx                  ; add intensity
  Mov  dl,[ecx]                 ; get shade color
                                ;
  Xor  ecx,ecx                  ;
  Dec  [_Count]                  ;
                                ;
  Mov  [edi-1],dl               ;
                                ;
  Jnz @@FillLoop                ;
  Pop  ebp                      ;
                                ;
 @@SkipFill:                    ;
                                ;
  Mov  edx,[_YAddress]           ; go to the next scanline
  Add  edx,_LineSize           ;
  Mov  [_YAddress],edx           ; YAddress += LineSize
                                ;
  Mov  eax,[Side&C]             ; increment the U span
  Add  eax,[Side&DC]            ;
  Mov  [Side&C],eax             ;
                                ;
  Mov  eax,[Side&U]             ; increment the U span
  Add  eax,[Side&DU]            ;
  Mov  [Side&U],eax             ;
                                ;
  Mov  eax,[Side&V]             ; increment the V span
  Add  eax,[Side&DV]            ;
  Mov  [Side&V],eax             ;
                                ;
  Mov  eax,[Left&X]             ;
  Mov  ebx,[Right&X]            ;
  Add  eax,[Left&DX]            ;
  Add  ebx,[Right&DX]           ;
  Mov  [Left&X],eax             ; LeftX += LeftDX
  Mov  [Right&X],ebx            ; RightX += RightDX
                                ;
  Jmp @@HeightLooper            ;
                                ;
 @@ExitHeight:                  ;
EndM                            ;
                                ;
;===========================================================================
; CullPolys
;===========================================================================
_GouraudTexturePolygonASM Proc Uses edi eax,_Triangle:DWord, _VirtualAddress:DWord, _LineSize:DWord
                                ;
  Pushad

  Mov edi,_Triangle            ;
                                ;
  Mov   eax,[edi].Texture       ;
  Mov   [_TextureOfsTable],eax   ;  grab and store the texture pointer
                                ;
include affasm.inc              ; include affine interpolation
                                ;
  Popad

  Ret                           ;
                                ;
_GouraudTexturePolygonASM EndP   ;


CODE ENDS
END


